@@ -183,37 +183,38 @@ sapwood_pixmap_get_for_file (const char *filename,
       {
 	GdkPixmap *pixmap  = NULL;
 	GdkBitmap *pixmask = NULL;
-
-	/* XXX X errors */
-
-	gdk_error_trap_push ();
+	int xerror;
 
 	if (rep.pixmap[i][j])
 	  {
+	    gdk_error_trap_push ();
 	    pixmap = gdk_pixmap_foreign_new (rep.pixmap[i][j]);
-	    if (!pixmap)
+	    gdk_flush ();
+	    if ((xerror = gdk_error_trap_pop ()) || !pixmap)
 	      {
-		g_warning ("%s: pixmap[%d][%d]: gdk_pixmap_foreign_new(%x) failed", 
-			   g_basename (filename), i, j, rep.pixmap[i][j]);
+		g_warning ("%s: pixmap[%d][%d]: gdk_pixmap_foreign_new(%x) failed, X error = %d",
+			   g_basename (filename), i, j, rep.pixmap[i][j], xerror);
+		if (pixmap)
+		  g_object_unref (pixmap);
+		pixmap = NULL;
 	      }
 	  }
 
 	if (rep.pixmask[i][j])
 	  {
+	    gdk_error_trap_push ();
 	    pixmask = gdk_pixmap_foreign_new (rep.pixmask[i][j]);
-	    if (!pixmask)
+	    gdk_flush ();
+	    if ((xerror = gdk_error_trap_pop ()) || !pixmask)
 	      {
-		g_warning ("%s: pixmask[%d][%d]: gdk_pixmap_foreign_new(%x) failed", 
-			   g_basename (filename), i, j, rep.pixmask[i][j]);
+		g_warning ("%s: pixmask[%d][%d]: gdk_pixmap_foreign_new(%x) failed, X error = %d", 
+			   g_basename (filename), i, j, rep.pixmask[i][j], xerror);
+		if (pixmask)
+		  g_object_unref (pixmask);
+		pixmask = NULL;
 	      }
 	  }
 
-	gdk_flush ();
-	if (gdk_error_trap_pop ())
-	  {
-	    g_warning ("Aieeee");
-	  }
-
 	if (pixmask && !pixmap)
 	  {
 	    g_warning ("%s: pixmask[%d][%d]: no pixmap", g_basename (filename), i, j);
@@ -324,39 +325,41 @@ sapwood_pixmap_render_rects_internal (SapwoodPixmap *self,
   xofs = draw_x - mask_x;
   yofs = draw_y - mask_y;
 
-  if (!mask_gc)
+  if (mask)
     {
-      values.fill = GDK_TILED;
-      mask_gc = gdk_gc_new_with_values (mask, &values, GDK_GC_FILL);
-    }
-
-  for (n = 0; n < n_rect; n++)
-    {
-      /* const */ GdkRectangle *dest = &rect[n].dest;
-      GdkRectangle              area;
-
-      if (!mask_required && clip_rect)
+      if (!mask_gc)
 	{
-	  if (!gdk_rectangle_intersect (dest, clip_rect, &area))
-	    continue;
-	}	  
-      else
-	area = *dest;
+	  values.fill = GDK_TILED;
+	  mask_gc = gdk_gc_new_with_values (mask, &values, GDK_GC_FILL);
+	}
 
-      if (rect[n].pixmap && rect[n].pixmask)
+      for (n = 0; n < n_rect; n++)
 	{
-	  values.tile = rect[n].pixmask;
-	  values.ts_x_origin = dest->x - xofs;
-	  values.ts_y_origin = dest->y - yofs;
-	  gdk_gc_set_values (mask_gc, &values, GDK_GC_TILE|GDK_GC_TS_X_ORIGIN|GDK_GC_TS_Y_ORIGIN);
+	  /* const */ GdkRectangle *dest = &rect[n].dest;
+	  GdkRectangle              area;
 
-	  gdk_draw_rectangle (mask, mask_gc, TRUE, area.x - xofs, area.y - yofs, area.width, area.height);
+	  if (!mask_required && clip_rect)
+	    {
+	      if (!gdk_rectangle_intersect (dest, clip_rect, &area))
+		continue;
+	    }	  
+	  else
+	    area = *dest;
 
-	  have_mask = TRUE;
+	  if (rect[n].pixmap && rect[n].pixmask)
+	    {
+	      values.tile = rect[n].pixmask;
+	      values.ts_x_origin = dest->x - xofs;
+	      values.ts_y_origin = dest->y - yofs;
+	      gdk_gc_set_values (mask_gc, &values, GDK_GC_TILE|GDK_GC_TS_X_ORIGIN|GDK_GC_TS_Y_ORIGIN);
+
+	      gdk_draw_rectangle (mask, mask_gc, TRUE, area.x - xofs, area.y - yofs, area.width, area.height);
+
+	      have_mask = TRUE;
+	    }
 	}
     }
 
-
   if (!draw_gc)
     {
       values.fill = GDK_TILED;
